from Products.CMFPlone.utils import base_hasattr
from Products.CMFCore.utils import getToolByName
from StringIO import StringIO
from Products.contentmigration.basemigrator.walker import CatalogWalker
from Products.contentmigration.inplace import InplaceCMFFolderMigrator
from plone.app.vocabularies.users import UsersSource
from plone.app.vocabularies.groups import GroupsSource
from zope.event import notify
from collective.groupspace.roles.browser.roles import LocalGroupSpacePASRolesChangeEvent
from Globals import PersistentMapping
from collective.groupspace.roles.localroles import setPolicyDefaultLocalRoles
from Products.CMFPlacefulWorkflow.PlacefulWorkflowTool import WorkflowPolicyConfig_id
from collective.groupspace.workflow.handlers import add_local_groupspace_workflow

class GrufSpacesMigrator(InplaceCMFFolderMigrator):
    """Choose the right product/package combination for your Plone site!
    
    If you don't have existing content, you don't need the GrufSpaces
    product, but can use collective.groupspace.content directly. 
    
        * http://pypi.python.org/pypi/collective.groupspace.content
    
    GrufSpaces and collective.groupspace.content are interchangeable. None of 
    the other collective.groupspace packages has a dependency on either one.
    
        * http://pypi.python.org/pypi/collective.groupspace.mail
    
        * http://pypi.python.org/pypi/collective.groupspace.workflow
        
        * http://pypi.python.org/pypi/collective.groupspace.roles

    Migration script dependency

    Please note that the migration script only works if you have 
    Products.contentmigration installed: 

        * http://pypi.python.org/pypi/Products.contentmigration

    Backup
    
    Backup you Data.fs before the migration. Make sure to keep a copy
    of any workflow you may have customized.

    Migration
    
    If you have existing GrufSpaces content, the easiest way to use the
    collective.groupspace suite is to follow these steps:
       
    1. Put Data.fs into the new Plone version

    2. Run the Plone migration in portal_migration

        http://localhost:8080/Plone/portal_migration/manage_migrate

    3. Add this External Method
    
        Id: GrufSpacesMigration
        Module Name: GrufSpaces.migration
        Function Name: migrate
    
    4. Run the migration by clicking on the Test tab here:
    
        http://localhost:8080/Plone/GrufSpacesMigration/manage_main
    """
    
    walkerClass = CatalogWalker

    src_portal_type = "GroupSpace"
    dst_portal_type = "GroupSpace"
    src_meta_type = "GroupSpace"
    dst_meta_type = "GroupSpace"
    
    def migrate(self):
        """
        """
        self.migrate_local_roles()
        self.migrate_local_workflow()
        
    def migrate_local_roles(self):
        # Only migrate if the user_roles and group_roles still have their 
        # initial values
        if self.old.user_roles is None and self.old.group_roles is None:
            print >> self.out, "Migrating local roles: %s" % self.old.absolute_url()
            self._migrate_local_roles()
        else:
            print >> self.out, "Local roles already migrated: %s" % self.old.absolute_url()
       
    def _migrate_local_roles(self):    
        # Let's collect the user and group roles we can then set if needed
        user_roles = {}
        group_roles = {}                

        for principal, roles in self.old.__ac_local_roles__.items():
            new_roles = []
            if 'GroupVisitor' in roles:
                new_roles.append('GroupReader')
            if 'GroupAdmin' in roles:
                new_roles.append('GroupAdmin')            
            if 'GroupMember' in roles:
                new_roles.append('GroupContributor')            
            if 'GroupLeader' in roles:
                new_roles.append('GroupAdmin')
                new_roles.append('GroupEditor')

            # Now we need to find out if the principal is a user or a group

            userssource = UsersSource(self.old)
            groupssource = GroupsSource(self.old)

            user = userssource.get(principal)
            if user is None:
                group = groupssource.get(principal)
                if group is None:
                    print >> self.out, "Can't find principal: %s" % principal
                else:
                    group_roles[principal] = list(new_roles)
            else:
                user_roles[principal] = list(new_roles)
                                
        if user_roles:
            # Set the user roles as expected by the ILocalGroupSpacePASRoles
            # interface
            self.old.user_roles = PersistentMapping()
            self.old.user_roles.update(user_roles)
            for principal, roles in user_roles.items():
                print >> self.out, "Roles for user %s: %s" % (principal, roles)
        
        if group_roles:
            # Set the group roles as expected by the ILocalGroupSpacePASRoles
            # interface
            self.old.group_roles = PersistentMapping()
            self.old.group_roles.update(group_roles)
            for principal, roles in group_roles.items():
                print >> self.out, "Roles for group %s: %s" % (principal, roles)

        if user_roles or group_roles:
            # In case there are some changes, trigger the event
            event = LocalGroupSpacePASRolesChangeEvent(self.old,
                                                       {}, 
                                                       user_roles, 
                                                       {}, 
                                                       group_roles)
            setPolicyDefaultLocalRoles(self.old, event)

    def migrate_local_workflow(self):
        # Only migrate if the workflow policy object is not already present
        if WorkflowPolicyConfig_id in self.old.objectIds():
            print >> self.out, "Local workflow already migrated: %s" % self.old.absolute_url()
        else:
            print >> self.out, "Migrating local workflow: %s" % self.old.absolute_url()
            self._migrate_local_workflow()

    def _migrate_local_workflow(self):
        add_local_groupspace_workflow(self.old, event=None)

def uninstall_portlets(portal, out):
    """Uninstall the GrufSpaces portlets in right slots"""


    from plone.portlets.interfaces import IPortletAssignmentMapping
    from plone.portlets.interfaces import IPortletManager
    from zope.component import getUtility
    from zope.component import getMultiAdapter

    rightcol = getUtility(IPortletManager, name=u'plone.rightcolumn', context=portal)
    right = getMultiAdapter((portal, rightcol,), IPortletAssignmentMapping, context=portal)
    leftcol = getUtility(IPortletManager, name=u'plone.leftcolumn', context=portal)
    left = getMultiAdapter((portal, leftcol,), IPortletAssignmentMapping, context=portal)

    grufspaces_portlets = (
           u'portlet_groups',
           u'portlet_visitor_groups',
           u'portlet_groupspaces_changes',
           u'portlet_groupspace',
           ) 

    changed = False
              
    for portlet in grufspaces_portlets:
        if portlet in right:
            print >> out, 'Uninstalling portlet:%s.' % portlet
            del right[portlet]
            changed = True
        if portlet in left:
            print >> out, 'Uninstalling portlet:%s.' % portlet
            del left[portlet]
        
    print >> out, 'GrufSpaces portlets uninstalled.'

def uninstall_configlet(self, out):
    # Uninstall configlets
    cp_tool = getToolByName(self, 'portal_controlpanel')
    uninstalled = True
    try:
        cp_tool.unregisterApplication('GrufSpaces')
    except:
        uninstalled = False
    if uninstalled:
        print >> out, 'GrufSpaces configlet uninstalled.'
    else:
        print >> out, 'GrufSpaces configlet already uninstalled.'

def uninstall_catalog_indices(self, out):

    # install catalog indices
    catalog = getToolByName(self, 'portal_catalog')
    
    indices = ['listGroupSpaceMemberUsers',
               'listGroupSpaceVisitorUsers',
               'listGroupSpaceMemberIds',
               'listGroupSpaceVisitorIds',
              ]

    remove = []

    for index in indices:              
        if index in catalog.indexes():
            remove.append(index)
    
    if remove:
        for index in remove:        
            print >> out, "Removing catalog index %s" % index
            catalog.manage_delIndex(index)
    else:
        print >> out, "No catalog indices to remove."
    
def uninstall_properties(self, out):
    if 'grufspaces_properties' in self.portal_properties.objectIds():
        self.portal_properties.manage_delObjects('grufspaces_properties')
        print >> out, "GrufSpaces property sheet uninstalled"
    else:
        print >> out, "GrufSpaces property sheet already uninstalled"

def uninstall_roles(self, out):
    roles = []

    # Custom roles installation
    if "GroupVisitor" in self.userdefined_roles():
        roles.append("GroupVisitor")
    if "GroupMember" in self.userdefined_roles():
        roles.append("GroupMember")
    if "GroupLeader" in self.userdefined_roles():
        roles.append("GroupLeader")

    if roles:
        self._delRoles(roles, None)
        print >> out, "GrufSpaces roles uninstalled: %s" % roles
    else:
        print >> out, "GrufSpaces roles already uninstalled."

def reinstall_grufspaces(self, out):
    q_in = self.portal_quickinstaller
    grufspaces = None
    for product in q_in.listInstalledProducts():
        if product['id'] == 'GrufSpaces':
            grufspaces = product

    grufspaces['productVersion'] = q_in.getProductVersion("GrufSpaces")
    
    if grufspaces:
        _reinstall_grufspaces(self, out, grufspaces)
    else:
        print >> out, "Can't find the GrufSpaces product"

def _reinstall_grufspaces(self, out, grufspaces):
    q_in = self.portal_quickinstaller
    if grufspaces['installedVersion'] != grufspaces['productVersion']:
        q_in.reinstallProducts(['GrufSpaces'])
        print >> out, "GrufSpaces reinstalled"
    else:
        print >> out, "GrufSpaces has already been reinstalled."

    
def refresh_catalog(portal, out):
    portal.portal_catalog.refreshCatalog(clear=1)
    print >> out, "Catalog refreshed"

def update_role_mappings(self, out):
    self.portal_workflow.updateRoleMappings()
    print >> out, "Role mappings updated."
                       
def migrate(self):
    """Run the migration"""

    out = StringIO()
    print >> out, "Starting migration"

    portal_url = getToolByName(self, 'portal_url')
    portal = portal_url.getPortalObject()

    print >> out, "\nReinstalling GrufSpaces"
    reinstall_grufspaces(portal, out)

    print >> out, "\nRefreshing the catalog"
    refresh_catalog(portal, out)

    print >> out, "\nUninstalling portlets"
    uninstall_portlets(portal, out)
    
    print >> out, "\nUninstalling configlet"
    uninstall_configlet(portal, out)

    print >> out, "\nUninstalling catalog indices"
    uninstall_catalog_indices(portal, out)
    
    print >> out, "\nUninstalling properties"
    uninstall_properties(portal, out)

    print >> out, "\nUninstalling roles"
    uninstall_roles(portal, out)

    print >> out, "\nMigrating the content"
    
    migrators = (GrufSpacesMigrator,)

    for migrator in migrators:
        migrator.out = out
        walker = migrator.walkerClass(portal, migrator)
        walker.go(out=out)
        print >> out, walker.getOutput()

    print >> out, "\nUpdating role mappings"
    update_role_mappings(portal, out)    

    print >> out, "\nRefreshing the catalog"
    refresh_catalog(portal, out)

    print >> out, "\nMigration finished"
    return out.getvalue()
